/*
 * Generic routines for single display/screen versions
 */

#include "gdk.h"
#include "gdkscreen-single.h"

typedef struct _GdkDisplaySingle GdkDisplaySingle;
typedef struct _GdkDisplaySingleClass GdkDisplaySingleClass;

#define GDK_TYPE_DISPLAY_SINGLE              (gdk_display_single_get_type())
#define GDK_DISPLAY_SINGLE(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_DISPLAY_SINGLE, GdkDisplaySingle))
#define GDK_DISPLAY_SINGLE_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_DISPLAY_SINGLE, GdkDisplaySingleClass))
#define GDK_IS_DISPLAY_SINGLE(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), GDK_TYPE_DISPLAY_SINGLE))
#define GDK_IS_DISPLAY_SINGLE_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GDK_TYPE_DISPLAY_SINGLE))
#define GDK_DISPLAY_SINGLE_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_DISPLAY_SINGLE, GdkDisplaySingleClass))

struct _GdkDisplaySingle
{
  GdkDisplay parent_instance;
};

struct _GdkDisplaySingleClass
{
  GdkDisplayClass parent_class;
};

static void                 gdk_display_single_class_init         (GdkDisplaySingleClass *class);
static gint                 gdk_display_single_get_n_screens      (GdkDisplay         *display);
static GdkScreen *          gdk_display_single_get_screen         (GdkDisplay         *display,
						                gint                screen_num);
G_CONST_RETURN static char *gdk_display_single_get_display_name   (GdkDisplay         *display);
static GdkScreen *          gdk_display_single_get_default_screen (GdkDisplay         *display);
static void                 gdk_display_single_finalize           (GObject            *object);

static gpointer parent_class = NULL;

GdkWindow *_gdk_parent_root = NULL;
GdkDisplay *_gdk_display = NULL;
GdkScreen  *_gdk_screen  = NULL;

GType
gdk_display_single_get_type ()
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
	{
	  sizeof (GdkDisplaySingleClass),
	  (GBaseInitFunc) NULL,
	  (GBaseFinalizeFunc) gdk_display_single_finalize,
	  (GClassInitFunc) gdk_display_single_class_init,
	  NULL,			/* class_finalize */
	  NULL,			/* class_data */
	  sizeof (GdkDisplaySingle),
	  0,			/* n_preallocs */
	  (GInstanceInitFunc) NULL,
	};
      
      object_type = g_type_register_static (GDK_TYPE_DISPLAY,
					    "GdkDisplaySingle",
					    &object_info, 0);
    }
  
  return object_type;
}

static void
gdk_display_single_class_init (GdkDisplaySingleClass * class)
{
  GdkDisplayClass *display_class = GDK_DISPLAY_CLASS (class);

  display_class->get_display_name = gdk_display_single_get_display_name;
  display_class->get_n_screens = gdk_display_single_get_n_screens;
  display_class->get_screen = gdk_display_single_get_screen;
  display_class->get_default_screen = gdk_display_single_get_default_screen;
  
  G_OBJECT_CLASS (class)->finalize = gdk_display_single_finalize;
  
  parent_class = g_type_class_peek_parent (class);
}

static void
gdk_display_single_finalize (GObject *object)
{
  G_OBJECT_CLASS (parent_class)->finalize (object);
}

static G_CONST_RETURN gchar*
gdk_display_single_get_display_name (GdkDisplay * display)
{
  GdkDisplaySingle *display_single = GDK_DISPLAY_SINGLE (display);
  
  return gdk_get_display ();
}

static gint
gdk_display_single_get_n_screens (GdkDisplay * display)
{
  GdkDisplaySingle *display_single = GDK_DISPLAY_SINGLE (display);
  
  return 1;
}

static GdkScreen *
gdk_display_single_get_screen (GdkDisplay *display,
			    gint        screen_num)
{
  GdkDisplaySingle *display_single = GDK_DISPLAY_SINGLE (display);

  g_return_val_if_fail (1 > screen_num, NULL);

  return _gdk_screen;
}

static GdkScreen *
gdk_display_single_get_default_screen (GdkDisplay * display)
{
  GdkDisplaySingle *display_single = GDK_DISPLAY_SINGLE (display);
  
  return _gdk_screen;
}


/*
 * Non member functions
 */

void
gdk_display_sync (GdkDisplay * display)
{
  g_return_if_fail (GDK_IS_DISPLAY_SINGLE (display));
}

void
_gdk_windowing_set_default_display (GdkDisplay *display)
{
  g_assert (_gdk_display == display);
}

GdkDisplay *
gdk_open_display (const gchar *display_name)
{
  GdkScreenSingle *screen_single;

  if (_gdk_display != NULL)
    return NULL; /* single display only */

  _gdk_display = g_object_new (GDK_TYPE_DISPLAY_SINGLE, NULL);
  _gdk_screen = g_object_new (GDK_TYPE_SCREEN_SINGLE, NULL);
  screen_single = GDK_SCREEN_SINGLE (_gdk_screen);
  screen_single->display = _gdk_display;

  gdk_set_default_display (_gdk_display);

  _gdk_visual_init ();
  _gdk_windowing_window_init ();
  _gdk_windowing_image_init ();
  _gdk_events_init ();
  _gdk_input_init ();
  _gdk_dnd_init ();

  return _gdk_display;
}

void
gdk_display_beep (GdkDisplay * display)
{
  g_return_if_fail (GDK_IS_DISPLAY (display));

  gdk_beep ();
}

void
gdk_display_keyboard_ungrab (GdkDisplay *display,
			     guint32     time)
{
  g_return_if_fail (display == _gdk_display);

  gdk_keyboard_ungrab (time);
}

void
gdk_display_pointer_ungrab (GdkDisplay *display,
			    guint32     time)
{
  g_return_if_fail (display == _gdk_display);

  gdk_pointer_ungrab (time);
}

gboolean
gdk_display_pointer_is_grabbed (GdkDisplay * display)
{
  g_return_val_if_fail (display == _gdk_display, FALSE);

  return gdk_pointer_is_grabbed ();
}

gboolean
gdk_event_send_client_message_for_display (GdkDisplay *display,
					   GdkEvent *event,
					   guint32 xid)
{
  g_return_val_if_fail (display == _gdk_display, FALSE);

  return gdk_event_send_client_message (event, xid);
}

GdkFont *
gdk_font_from_description_for_display (GdkDisplay           *display,
				       PangoFontDescription *font_desc)
{
  g_return_val_if_fail (display == _gdk_display, NULL);

  return gdk_font_from_description (font_desc);
}

GdkFont *
gdk_font_load_for_display (GdkDisplay  *display, 
			   const gchar *font_name)
{
  g_return_val_if_fail (display == _gdk_display, NULL);

  return gdk_font_load (font_name);
}

GdkKeymap*
gdk_keymap_get_for_display (GdkDisplay *display)
{
  g_return_val_if_fail (display == _gdk_display, NULL);

  return gdk_keymap_get_default ();
}

GdkWindow *
gdk_selection_owner_get_for_display (GdkDisplay *display,
				     GdkAtom     selection)
{
  g_return_val_if_fail (display == _gdk_display, NULL);

  return gdk_selection_owner_get (selection);
}

gboolean
gdk_selection_owner_set_for_display (GdkDisplay *display,
				     GdkWindow  *owner,
				     GdkAtom     selection,
				     guint32     time, 
				     gboolean    send_event)
{
  g_return_val_if_fail (display == _gdk_display, FALSE);

  return gdk_selection_owner_set (owner, selection, time, send_event);
}

void
gdk_selection_send_notify_for_display (GdkDisplay *display,
				       guint32     requestor,
				       GdkAtom     selection,
				       GdkAtom     target,
				       GdkAtom     property, 
				       guint32     time)
{
  g_return_if_fail (display == _gdk_display);

  gdk_selection_send_notify (requestor, selection, 
                             target, property, time);
}

gint 
gdk_text_property_to_utf8_list_for_display (GdkDisplay    *display,
					    GdkAtom        encoding,
					    gint           format,
					    const guchar  *text,
					    gint           length,
					    gchar       ***list)
{
  g_return_val_if_fail (display == _gdk_display, 0);

  return gdk_text_property_to_utf8_list (encoding, format, 
                                         text, length, list);
}

gboolean
gdk_utf8_to_compound_text_for_display (GdkDisplay  *display,
				       const gchar *str,
				       GdkAtom     *encoding,
				       gint        *format,
				       guchar     **ctext,
				       gint        *length)
{
  g_return_val_if_fail (display == _gdk_display, FALSE);

  return gdk_utf8_to_compound_text (str, encoding, format, 
                                    ctext, length);
}

GdkWindow *
gdk_window_foreign_new_for_display (GdkDisplay     *display,
				    GdkNativeWindow anid)
{
  g_return_val_if_fail (display == _gdk_display, NULL);

  return gdk_window_foreign_new (anid);
}

GdkWindow *
gdk_window_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
{
  g_return_val_if_fail (display == _gdk_display, NULL);

  return gdk_window_lookup (anid);
}