#include <glib.h>

#include "gdk.h"
#include "gdkscreen-single.h"
#include "gdkinternals.h"

#ifdef GDK_MULTIHEAD_SAFE
#error "This is for single display/screnn gdk systems"
#endif

static void         gdk_screen_single_class_init       (GdkScreenSingleClass *klass);
static GdkDisplay * gdk_screen_single_get_display           (GdkScreen             *screen);
static gint         gdk_screen_single_get_width             (GdkScreen             *screen);
static gint         gdk_screen_single_get_height            (GdkScreen             *screen);
static gint         gdk_screen_single_get_width_mm          (GdkScreen             *screen);
static gint         gdk_screen_single_get_height_mm         (GdkScreen             *screen);
static gint         gdk_screen_single_get_default_depth     (GdkScreen             *screen);
static GdkWindow *  gdk_screen_single_get_root_window       (GdkScreen             *screen);
static gint         gdk_screen_single_get_screen_num        (GdkScreen             *screen);
static GdkColormap *gdk_screen_single_get_default_colormap  (GdkScreen             *screen);
static void         gdk_screen_single_set_default_colormap  (GdkScreen             *screen,
							  GdkColormap           *colormap);
static GdkWindow *  gdk_screen_single_get_window_at_pointer (GdkScreen             *screen,
							  gint                  *win_x,
							  gint                  *win_y);
static void         gdk_screen_single_finalize		 (GObject		*object);

static gint          gdk_screen_single_get_n_monitors        (GdkScreen       *screen);
static void          gdk_screen_single_get_monitor_geometry  (GdkScreen       *screen,
							   gint             num_monitor,
							   GdkRectangle    *dest);

GType gdk_screen_single_get_type ();
static gpointer parent_class = NULL;

struct _GdkScreenSingleClass
{
  GdkScreenClass parent_class;
};


GType
gdk_screen_single_get_type ()
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
	{
	  sizeof (GdkScreenSingleClass),
	  (GBaseInitFunc) NULL,
	  (GBaseFinalizeFunc) gdk_screen_single_finalize,
	  (GClassInitFunc) gdk_screen_single_class_init,
	  NULL,			/* class_finalize */
	  NULL,			/* class_data */
	  sizeof (GdkScreenSingle),
	  0,			/* n_preallocs */
	  (GInstanceInitFunc) NULL,
	};
      object_type = g_type_register_static (GDK_TYPE_SCREEN,
					    "GdkScreenSingle",
					    &object_info, 0);
    }
  return object_type;
}

void
gdk_screen_single_class_init (GdkScreenSingleClass * klass)
{
  GdkScreenClass *screen_class = GDK_SCREEN_CLASS (klass);
  
  screen_class->get_display = gdk_screen_single_get_display;
  screen_class->get_width = gdk_screen_single_get_width;
  screen_class->get_height = gdk_screen_single_get_height;
  screen_class->get_width_mm = gdk_screen_single_get_width_mm;
  screen_class->get_height_mm = gdk_screen_single_get_height_mm;
  screen_class->get_root_depth = gdk_screen_single_get_default_depth;
  screen_class->get_screen_num = gdk_screen_single_get_screen_num;
  screen_class->get_root_window = gdk_screen_single_get_root_window;
  screen_class->get_default_colormap = gdk_screen_single_get_default_colormap;
  screen_class->set_default_colormap = gdk_screen_single_set_default_colormap;
  screen_class->get_window_at_pointer = gdk_screen_single_get_window_at_pointer;
  screen_class->get_n_monitors = gdk_screen_single_get_n_monitors;
  screen_class->get_monitor_geometry = gdk_screen_single_get_monitor_geometry;
  
  G_OBJECT_CLASS (klass)->finalize = gdk_screen_single_finalize;
  parent_class = g_type_class_peek_parent (klass);
}

static GdkDisplay *
gdk_screen_single_get_display (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return screen_Single->display;
}

static gint
gdk_screen_single_get_width (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return gdk_screen_width ();
}

static gint
gdk_screen_single_get_height (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return gdk_screen_height ();
}

static gint
gdk_screen_single_get_width_mm (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return gdk_screen_width_mm ();
}

static gint
gdk_screen_single_get_height_mm (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return gdk_screen_height_mm ();
}

static gint
gdk_screen_single_get_default_depth (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return gdk_visual_get_best_depth ();
}

static gint
gdk_screen_single_get_screen_num (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);
  
  return 1;
}

static GdkWindow *
gdk_screen_single_get_root_window (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return _gdk_parent_root;
}

static GdkColormap *
gdk_screen_single_get_default_colormap (GdkScreen *screen)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);

  return gdk_window_get_colormap (_gdk_parent_root);
}

static void
gdk_screen_single_set_default_colormap (GdkScreen *screen,
				     GdkColormap * colormap)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (screen);
  
  gdk_window_set_colormap (_gdk_parent_root, colormap);
}

static GdkWindow *
gdk_screen_single_get_window_at_pointer (GdkScreen *screen,
				      gint      *win_x,
				      gint      *win_y)
{
  return gdk_window_at_pointer (win_x, win_y);
}

static void
gdk_screen_single_finalize (GObject *object)
{
  GdkScreenSingle *screen_Single = GDK_SCREEN_SINGLE (object);

  /* ... ? */

  G_OBJECT_CLASS (parent_class)->finalize (object);
}

static gint 
gdk_screen_single_get_n_monitors (GdkScreen *screen)
{
  g_return_val_if_fail (GDK_IS_SCREEN (screen), 1);

  return 1;
}

static void
gdk_screen_single_get_monitor_geometry (GdkScreen    *screen, 
				     gint          num_monitor,
				     GdkRectangle *dest)
{
  dest->x = 0;
  dest->y = 0;
  dest->width = gdk_screen_width ();
  dest->height = gdk_screen_height ();
}

/**
 * gdk_single_screen_get_screen_number:
 * @screen: a #GdkScreen.
 * @returns: the position of @screen among the screens of
 *   its display.
 *
 * Returns the index of a #GdkScreen.
 */
int
gdk_single_screen_get_screen_number (GdkScreen *screen)
{
  return 1;
}


/*
 * ...
 */
GdkVisual *
gdk_screen_get_system_visual (GdkScreen * screen)
{
  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);

  return gdk_visual_get_system ();
}

GList *
gdk_screen_list_visuals (GdkScreen *screen)
{
  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);

  return gdk_list_visuals ();
}

GdkColormap *
gdk_screen_get_system_colormap (GdkScreen *screen)
{
  return gdk_colormap_get_system ();
}


GdkScreen *
gdk_cursor_get_screen (GdkCursor *cursor)
{
  return _gdk_screen;
}

GdkCursor*
gdk_cursor_new_for_screen (GdkScreen    *screen,
			   GdkCursorType cursor_type)
{
  g_return_val_if_fail (GDK_IS_SCREEN_SINGLE (screen), NULL);

  return gdk_cursor_new (cursor_type);
}

PangoContext *
gdk_pango_context_get_for_screen (GdkScreen *screen)
{
  g_return_val_if_fail (screen == _gdk_screen, NULL);

  return gdk_pango_context_get ();
}

gboolean
gdk_screen_get_setting (GdkScreen   *screen,
			const gchar *name,
			GValue      *value)
{
  g_return_val_if_fail (screen == _gdk_screen, FALSE);

  return gdk_setting_get (name, value);
}